import 'id.dart';
import 'package:json_annotation/json_annotation.dart';

@JsonSerializable()
abstract class Document<T extends ID> {
  T _id;
  Map<String, Object> fragments;
  String self;

  T getId() {
    return _id;
  }

  void setId(T id) {
    _id = id;
  }

  void set(String propertyName, Object object) {
    _setProperty(object, propertyName);
  }

  Object get(String propertyName) {
    if (fragments.containsKey(propertyName)) {
      return fragments[propertyName];
    } else {
      return null;
    }
  }

  Document();

  Document.withID(this._id);

  Document.withFragments(this._id, Map<String, Object> fragments) {
    _add(fragments);
  }

  void _add(Map<String, Object> fragments) {
    if (fragments == null) {
      return;
    }
    for (var fragment in fragments.entries) {
      _setProperty(fragment.key, fragment.value);
    }
  }

  void _setProperty(String propertyName, Object object) {
    fragments.putIfAbsent(propertyName, object);
  }
}
