

import 'package:c8y_client/src/core/model/id.dart';
import 'package:c8y_client/src/inventory/model/inventory_api.dart';
import 'package:c8y_client/src/inventory/model/managed_object_collention.dart';
import 'package:c8y_client/src/rest/representation/inventory/inventory_media_type.dart';
import 'package:c8y_client/src/rest/representation/inventory/managed_object_representation.dart';
import 'package:c8y_client/src/sdk/client/rest_connector.dart';

import 'managed_object_collection_impl.dart';

class Inventory implements InventoryApi {
  final RestConnector _restConnector;
  final int _pageSize;
  final String _path = '/inventory/managedObjects';
  final String _totalPages = 'withTotalPages=true';
  String _url;

  Inventory(this._restConnector, this._pageSize, String host) {
    _url = host + _path;
  }

  @override
  Future<ManagedObjectRepresentation> create(
      ManagedObjectRepresentation managedObject) async {
    var mo = await _restConnector.post(_url, InventoryMediaType.MANAGED_OBJECT,
        ManagedObjectRepresentation.fromJson);
    return mo;
  }

  @override
  void delete(ID id) {
    // TODO: implement delete
  }

  @override
  Future<ManagedObjectRepresentation> getManagedObject(ID id) async {
    return await _restConnector.get(
        _url + '/' + id.value,
        InventoryMediaType.MANAGED_OBJECT,
        ManagedObjectRepresentation.fromJson);
  }

  @override
  Future<ManagedObjectRepresentation> update(
      ManagedObjectRepresentation managedObject) async {
    // TODO: implement update
    return null;
  }

  @override
  ManagedObjectCollection managedObjects() {
    return ManagedObjectCollectionImpl(
        _restConnector, _url + '?' + _totalPages, _pageSize);
  }

//  Future<List<ManagedObjectRepresentation>> list(
//      Map<String, Object> queryParams) async {
//    var pagedCollection =
//        await ManagedObjectCollectionImpl(_restConnector, path + '?', _pageSize)
//            .get(queryParams);
//    var result = pagedCollection.managedObjects;
//    await for (var value in pagedCollection.list()) {
//      result.addAll(value);
//    }
//    return result;
//  }
  @override
  Future<List<ManagedObjectRepresentation>> list(
      Map<String, Object> queryParams) async {
    var pagedCollection = await ManagedObjectCollectionImpl(
            _restConnector, _url + '?' + _totalPages, _pageSize)
        .get(queryParams);
    return (await pagedCollection.allPages()).managedObjects;
  }
}
