import 'dart:collection';
import '../../core/model/id.dart';
import 'package:c8y_client/src/core/model/document.dart';
import 'package:json_annotation/json_annotation.dart';

class MyManagedObject<T extends ID> extends Document<T> {
  final String BINARY_MARKER = "c8y_IsBinary";
  final String GROUP_MARKER = "c8y_IsDeviceGroup";
  final String DEVICE_MARKER = "c8y_IsDevice";
  final String GROUP_TYPE = "c8y_DeviceGroup";
  final String SUBGROUP_TYPE = "c8y_DeviceSubgroup";
  final String SMART_RULE = "c8y_SmartRule";
  final String PRIVATE_SMART_RULE = "c8y_PrivateSmartRule";
  final String TENANT_TYPE = "c8y_Tenant";
  final String ATTACHMENT_MARKER_FRAGMENT = "c8y_applications_storage";
  final String CONTENT_TYPE = "contentType";
  final String LENGTH = "length";
  final String FRAGMENTS = "_fragments";
  final String SUPPORTED_MEASUREMENTS_PROPERTY_NAME = "supportedMeasurements";
  final String DYNAMIC_GROUP_TYPE = "c8y_DynamicGroup";
  final String DYNAMIC_GROUP_DEVICE_QUERY = "c8y_DeviceQueryString";
  final String EXTERNAL_IDS = "externalIds";
  final String NAME = "name";
  final String TYPE = "type";
  final String OWNER = "owner";
  final String LAST_UPDATED_DATE_TIME = "lastUpdated";
  final String CHILD_ASSETS = "childAssets";
  final String CHILD_DEVICES = "childDevices";
  final String CHILD_ADDITIONS = "childAdditions";

  Set<T> _childDevices =  LinkedHashSet<T>();
  Set<T> _childAssets =  LinkedHashSet<T>();
  Set<T> _childAdditions =  LinkedHashSet<T>();
  var _additionParents =  LinkedHashSet<MyManagedObject<T>>();
  var _assetParents =  LinkedHashSet<MyManagedObject<T>>();
  var _deviceParents =  LinkedHashSet<MyManagedObject<T>>();
  @JsonKey(name: 'owner')
  String _owner;
  String _type;
  String _name;
  DateTime _creationTime;
  DateTime _lastUpdated;

  @override
  String get owner {
    return _owner;
  }

  set owner(String owner) => _owner = owner;

  MyManagedObject();

  MyManagedObject.withID(T id) : super.withID(id);

  MyManagedObject.full(
      T id,
      this._type,
      this._name,
      this._lastUpdated,
      this._owner,
      this._childAssets,
      this._childDevices,
      this._childAdditions,
      this._assetParents,
      this._deviceParents,
      this._additionParents,
      Map<String, Object> fragments)
      : super.withFragments(id, fragments);

  factory MyManagedObject.fromJson(Map<String, dynamic> json) {
    var self = json['self'] as String;
    json.remove('self');
    var owner = json['owner'] as String;
    json.remove('owner');

    return MyManagedObject<T>()
      ..fragments = json
      ..self = self
      ..owner = owner;
  }
}
